class MaveController

	def initialize(params)
		@configs = params[:CONFIGS]

		# 各モデル生成
		@models = {}
		@models[:STATUS]	= MaveStatus.new({:CONFIGS => @configs}); $status = @models[:STATUS] # デバッグ用
		@models[:PREVIEW]	= MavePreview.new({:CONFIGS => @configs})
		@models[:FOLDERS]	= MaveFolders.new({:CONFIGS => @configs})
		@models[:ACCOUNTS]	= MaveAccounts.new({:CONFIGS => @configs})

		# 各フォルダ初期化
		@models[:ACCOUNTS].each {|account|
			[:INBOX_FOLDER, :DRAFT_FOLDER, :OUTBOX_FOLDER, :SENT_FOLDER, :TRASH_FOLDER].each {|folder|
				@models[:FOLDERS].open_folder(account[folder])
			}
		}

		# 代表ビュー生成
		@views = MaveViews.new(@models)

		@actions = {												# コントローラ自ら実行できるアクション一覧
			:mk_global_fetch_mail_pop		=> method(:fetch_mail_pop),
			:mk_global_send_mail_smtp		=> method(:send_mail_smtp),
			:mk_global_toggle_what_charset	=> method(:toggle_what_charset),
			:mk_global_kill_mave			=> method(:kill_mave),
		}
	end

	def action(command)
		(it = @actions[command]) ? it.call : nil
	end

	#-----------------------------------------------------------
	#
	#	メイン
	#
	def main
		begin
			loop {
				@views.clean
				@views.update									# 代表ビューに再表示を命令
				command = @views.ready(@views.active.name)		# アクティブビュー(キーマップ)を指定して、キー入力待ち
				@views.active.action(command) or				# アクティブビューにコマンドを処理させる
							@views.action(command) or			# 代表ビューにコマンドを処理させる
										action(command)			# ビューが実装してないコマンドは、コントローラ自ら処理する
			}
		rescue
			@views.close
			print $!.message + "\n"
			print $!.backtrace.join("\n")
		end
	end

	#-----------------------------------------------------------
	#
	#	メール受信(POP)
	#
	def fetch_mail_pop
		@pop_directory = MaveDirectory.new({:CONFIGS => @configs, :PATH => @configs[:POP_DIRECTORY]}) unless(@pop_directory)
		n_mail = 0; n_mail_max = 9999
		@models[:ACCOUNTS].each {|account|
			@models[:STATUS].log([_("Account '%1$s' connecting POP server '%2$s'..."), account.name, account.pop_server])
			begin
				account.pop {|popmail|
					@models[:STATUS].log([_(popmail)]) and next if(popmail.is_a?(String))
					n_mail_max = popmail and next if(popmail.is_a?(Integer))
					halfname = @pop_directory.create_mailfile {|fh|
						popmail.pop {|line|
							fh.write(line)
						}
					}
					fullname = @pop_directory.path + '/' + halfname
					mail = MaveMail.new({:CONFIGS => @configs, :FILE => File.new(fullname)})

					@models[:FOLDERS].each {|folder|
						if(folder.bind?(mail))
							folder.add_mail(mail)
							@pop_directory.delete(halfname)
						end
					}

					n_mail += 1
					@models[:STATUS].log([_('Popped (%1$d/%2$d) from [%3$s] [%4$s]'), n_mail, n_mail_max, mail.from.decode_mh, mail.subject.decode_mh])
				}
			rescue
				@models[:STATUS].log([_('Failed to pop mail. reason=[%s]'), $!.message])
			end
			@models[:STATUS].log([_('%1$s mail%2$s popped.'), (it = n_mail) == 0 ? 'no' : it.to_s, it == 1 ? '' : 's'])
		}
	end

	#-----------------------------------------------------------
	#
	#	メール送信(SMTP)
	#
	def send_mail_smtp
		#### 未実装
	end

	#-----------------------------------------------------------
	#
	#	キャラクタセット情報の表示、切り替え(デバッグ用)
	#
	def toggle_what_charset
		state = MaveMail.toggle_what_charset
		@models[:STATUS].log(['what charset: %s', state.to_s])
	end

	#-----------------------------------------------------------
	#
	#	終了する
	#
	def kill_mave
		raise 'Quit.'
	end
end

__END__

